@@ -17,11 +17,13 @@ from oslo_log import log
 from oslo_messaging import exceptions as oslo_msg_exc
 from oslo_utils import excutils
 from oslo_versionedobjects import exception as oslo_vo_exc
-from wsme import types as wtypes
 
+from ironic.api import types as atypes
 from ironic.common import exception
 from ironic.common.i18n import _
+from ironic.objects import allocation as allocation_objects
 from ironic.objects import chassis as chassis_objects
+from ironic.objects import deploy_template as deploy_template_objects
 from ironic.objects import fields
 from ironic.objects import node as node_objects
 from ironic.objects import notification
@@ -35,8 +37,12 @@ CONF = cfg.CONF
 
 
 CRUD_NOTIFY_OBJ = {
+    'allocation': (allocation_objects.AllocationCRUDNotification,
+                   allocation_objects.AllocationCRUDPayload),
     'chassis': (chassis_objects.ChassisCRUDNotification,
                 chassis_objects.ChassisCRUDPayload),
+    'deploytemplate': (deploy_template_objects.DeployTemplateCRUDNotification,
+                       deploy_template_objects.DeployTemplateCRUDPayload),
     'node': (node_objects.NodeCRUDNotification,
              node_objects.NodeCRUDPayload),
     'port': (port_objects.PortCRUDNotification,
@@ -65,8 +71,8 @@ def _emit_api_notification(context, obj, action, level, status, **kwargs):
     :param kwargs: kwargs to use when creating the notification payload.
     """
     resource = obj.__class__.__name__.lower()
-    # value wsme.Unset can be passed from API representation of resource
-    extra_args = {k: (v if v != wtypes.Unset else None)
+    # value atypes.Unset can be passed from API representation of resource
+    extra_args = {k: (v if v != atypes.Unset else None)
                   for k, v in kwargs.items()}
     try:
         try:
