@@ -20,7 +20,6 @@ from http import client as http_client
 
 from ironic_lib.exception import IronicException
 from oslo_log import log as logging
-import wsme
 
 from ironic.common.i18n import _
 
@@ -298,6 +297,10 @@ class InvalidIPv4Address(IronicException):
     _msg_fmt = _("Invalid IPv4 address %(ip_address)s.")
 
 
+class InvalidIPAddress(IronicException):
+    _msg_fmt = _("Invalid IP address %(ip_address)s.")
+
+
 class FailedToUpdateMacOnPort(IronicException):
     _msg_fmt = _("Update MAC address on port: %(port_id)s failed.")
 
@@ -380,7 +383,7 @@ class ImageNotAuthorized(NotAuthorized):
     _msg_fmt = _("Not authorized for image %(image_id)s.")
 
 
-class InvalidImageRef(Invalid):
+class InvalidImageRef(InvalidParameterValue):
     _msg_fmt = _("Invalid image href %(image_href)s.")
 
 
@@ -713,8 +716,21 @@ class IBMCConnectionError(IBMCError):
     _msg_fmt = _("IBMC connection failed for node %(node)s: %(error)s")
 
 
-class ClientSideError(wsme.exc.ClientSideError):
-    pass
+class ClientSideError(RuntimeError):
+    def __init__(self, msg=None, status_code=400, faultcode='Client'):
+        self.msg = msg
+        self.code = status_code
+        self.faultcode = faultcode
+        super(ClientSideError, self).__init__(self.faultstring)
+
+    @property
+    def faultstring(self):
+        if self.msg is None:
+            return str(self)
+        elif isinstance(self.msg, str):
+            return self.msg
+        else:
+            return str(self.msg)
 
 
 class NodeIsRetired(Invalid):
@@ -725,3 +741,80 @@ class NodeIsRetired(Invalid):
 class NoFreeIPMITerminalPorts(TemporaryFailure):
     _msg_fmt = _("Unable to allocate a free port on host %(host)s for IPMI "
                  "terminal, not enough free ports.")
+
+
+class InvalidInput(ClientSideError):
+    def __init__(self, fieldname, value, msg=''):
+        self.fieldname = fieldname
+        self.value = value
+        super(InvalidInput, self).__init__(msg)
+
+    @property
+    def faultstring(self):
+        return _(
+            "Invalid input for field/attribute %(fieldname)s. "
+            "Value: '%(value)s'. %(msg)s"
+        ) % {
+            'fieldname': self.fieldname,
+            'value': self.value,
+            'msg': self.msg
+        }
+
+
+class UnknownArgument(ClientSideError):
+    def __init__(self, argname, msg=''):
+        self.argname = argname
+        super(UnknownArgument, self).__init__(msg)
+
+    @property
+    def faultstring(self):
+        return _('Unknown argument: "%(argname)s"%(msg)s') % {
+            'argname': self.argname,
+            'msg': self.msg and ": " + self.msg or ""
+        }
+
+
+class MissingArgument(ClientSideError):
+    def __init__(self, argname, msg=''):
+        self.argname = argname
+        super(MissingArgument, self).__init__(msg)
+
+    @property
+    def faultstring(self):
+        return _('Missing argument: "%(argname)s"%(msg)s') % {
+            'argname': self.argname,
+            'msg': self.msg and ": " + self.msg or ""
+        }
+
+
+class UnknownAttribute(ClientSideError):
+    def __init__(self, fieldname, attributes, msg=''):
+        self.fieldname = fieldname
+        self.attributes = attributes
+        self.msg = msg
+        super(UnknownAttribute, self).__init__(self.msg)
+
+    @property
+    def faultstring(self):
+        error = _("Unknown attribute for argument %(argn)s: %(attrs)s")
+        if len(self.attributes) > 1:
+            error = _("Unknown attributes for argument %(argn)s: %(attrs)s")
+        str_attrs = ", ".join(self.attributes)
+        return error % {'argn': self.fieldname, 'attrs': str_attrs}
+
+    def add_fieldname(self, name):
+        """Add a fieldname to concatenate the full name.
+
+        Add a fieldname so that the whole hierarchy is displayed. Successive
+        calls to this method will prepend ``name`` to the hierarchy of names.
+        """
+        if self.fieldname is not None:
+            self.fieldname = "{}.{}".format(name, self.fieldname)
+        else:
+            self.fieldname = name
+        super(UnknownAttribute, self).__init__(self.msg)
+
+
+class AgentInProgress(IronicException):
+    _msg_fmt = _('Node %(node)s command "%(command)s" failed. Agent is '
+                 'presently executing a command. Error %(error)s')
