@@ -104,7 +104,7 @@ def update_neutron_port(context, port_id, attrs, client=None):
     return client.update_port(port_id, **attrs)
 
 
-def unbind_neutron_port(port_id, client=None, context=None):
+def unbind_neutron_port(port_id, client=None, context=None, reset_mac=True):
     """Unbind a neutron port
 
     Remove a neutron port's binding profile and host ID so that it returns to
@@ -113,6 +113,7 @@ def unbind_neutron_port(port_id, client=None, context=None):
     :param port_id: Neutron port ID.
     :param client: Optional a Neutron client object.
     :param context: request context
+    :param reset_mac: reset mac address
     :type context: ironic.common.context.RequestContext
     :raises: NetworkError
     """
@@ -126,7 +127,8 @@ def unbind_neutron_port(port_id, client=None, context=None):
         #   Exception PortBound will be raised by neutron as it refuses to
         #   update the mac address of a bound port if we attempt to unbind and
         #   reset the mac in the same call.
-        update_neutron_port(context, port_id, attrs_reset_mac, client)
+        if reset_mac:
+            update_neutron_port(context, port_id, attrs_reset_mac, client)
     # NOTE(vsaienko): Ignore if port was deleted before calling vif detach.
     except openstack_exc.ResourceNotFound:
         LOG.info('Port %s was not found while unbinding.', port_id)
@@ -342,7 +344,8 @@ def add_ports_to_network(task, network_uuid, security_groups=None):
                 wait_for_host_agent(
                     client, update_port_attrs['binding:host_id'])
             port = client.create_port(**port_attrs)
-            update_neutron_port(task.context, port.id, update_port_attrs)
+            port = update_neutron_port(task.context, port.id,
+                                       update_port_attrs)
             if CONF.neutron.dhcpv6_stateful_address_count > 1:
                 _add_ip_addresses_for_ipv6_stateful(task.context, port, client)
             if is_smart_nic:
