@@ -398,18 +398,42 @@ def cleanup_cleanwait_timeout(task):
                     "check if the ramdisk responsible for the cleaning is "
                     "running on the node. Failed on step %(step)s.") %
                   {'step': task.node.clean_step})
+    logmsg = ("Cleaning for node %(node)s failed. %(error)s" %
+              {'node': task.node.uuid, 'error': last_error})
     # NOTE(rloo): this is called from the periodic task for cleanwait timeouts,
     # via the task manager's process_event(). The node has already been moved
     # to CLEANFAIL, so the error handler doesn't need to set the fail state.
-    cleaning_error_handler(task, msg=last_error, set_fail_state=False)
+    cleaning_error_handler(task, logmsg, errmsg=last_error,
+                           set_fail_state=False)
 
 
-def cleaning_error_handler(task, msg, tear_down_cleaning=True,
-                           set_fail_state=True):
-    """Put a failed node in CLEANFAIL and maintenance."""
+def cleaning_error_handler(task, logmsg, errmsg=None, traceback=False,
+                           tear_down_cleaning=True, set_fail_state=True,
+                           set_maintenance=None):
+    """Put a failed node in CLEANFAIL and maintenance (if needed).
+
+    :param task: a TaskManager instance.
+    :param logmsg: Message to be logged.
+    :param errmsg: Message for the user. Optional, if not provided `logmsg` is
+        used.
+    :param traceback: Whether to log a traceback. Defaults to False.
+    :param tear_down_cleaning: Whether to clean up the PXE and DHCP files after
+        cleaning. Default to True.
+    :param set_fail_state: Whether to set node to failed state. Default to
+        True.
+    :param set_maintenance: Whether to set maintenance mode. If None,
+        maintenance mode will be set if and only if a clean step is being
+        executed on a node.
+    """
+    if set_maintenance is None:
+        set_maintenance = bool(task.node.clean_step)
+
+    errmsg = errmsg or logmsg
+    LOG.error(logmsg, exc_info=traceback)
     node = task.node
-    node.fault = faults.CLEAN_FAILURE
-    node.maintenance = True
+    if set_maintenance:
+        node.fault = faults.CLEAN_FAILURE
+        node.maintenance = True
 
     if tear_down_cleaning:
         try:
@@ -418,9 +442,7 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
             msg2 = ('Failed to tear down cleaning on node %(uuid)s, '
                     'reason: %(err)s' % {'err': e, 'uuid': node.uuid})
             LOG.exception(msg2)
-            msg = _('%s. Also failed to tear down cleaning.') % msg
-        else:
-            LOG.error(msg)
+            errmsg = _('%s. Also failed to tear down cleaning.') % errmsg
 
     if node.provision_state in (
             states.CLEANING,
@@ -441,10 +463,10 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
     # For manual cleaning, the target provision state is MANAGEABLE, whereas
     # for automated cleaning, it is AVAILABLE.
     manual_clean = node.target_provision_state == states.MANAGEABLE
-    node.last_error = msg
+    node.last_error = errmsg
     # NOTE(dtantsur): avoid overwriting existing maintenance_reason
-    if not node.maintenance_reason:
-        node.maintenance_reason = msg
+    if not node.maintenance_reason and set_maintenance:
+        node.maintenance_reason = errmsg
     node.save()
 
     if set_fail_state and node.provision_state != states.CLEANFAIL:
@@ -1013,6 +1035,11 @@ def agent_is_alive(node, timeout=None):
     :param node: A node object.
     :param timeout: Heartbeat timeout, defaults to `fast_track_timeout`.
     """
+    # If no agent_url is present then we have powered down since the
+    # last agent heartbeat
+    if not node.driver_internal_info.get('agent_url'):
+        return False
+
     return value_within_timeout(
         node.driver_internal_info.get('agent_last_heartbeat'),
         timeout or CONF.deploy.fast_track_timeout)
@@ -1136,6 +1163,8 @@ def add_secret_token(node, pregenerated=False):
     i_info['agent_secret_token'] = token
     if pregenerated:
         i_info['agent_secret_token_pregenerated'] = True
+    else:
+        i_info.pop('agent_secret_token_pregenerated', None)
     node.driver_internal_info = i_info
 
 
@@ -1284,3 +1313,34 @@ def store_agent_certificate(node, agent_verify_ca):
         LOG.debug('Saved the custom certificate for node %(node)s to %(file)s',
                   {'node': node.uuid, 'file': fname})
         return fname
+
+
+def node_cache_vendor(task):
+    """Cache the vendor if it can be detected."""
+    properties = task.node.properties
+    if properties.get('vendor'):
+        return  # assume that vendors don't change on fly
+
+    try:
+        # We have no vendor stored, so we'll go ahead and
+        # call to store it.
+        vendor = task.driver.management.detect_vendor(task)
+        if not vendor:
+            return
+
+        # This function may be called without an exclusive lock, so get one
+        task.upgrade_lock(purpose='caching node vendor')
+    except exception.UnsupportedDriverExtension:
+        return
+    except Exception as exc:
+        LOG.warning('Unexpected exception when trying to detect vendor '
+                    'for node %(node)s. %(class)s: %(exc)s',
+                    {'node': task.node.uuid,
+                     'class': type(exc).__name__, 'exc': exc},
+                    exc_info=not isinstance(exc, exception.IronicException))
+        return
+
+    props = task.node.properties
+    props['vendor'] = vendor
+    task.node.properties = props
+    task.node.save()
