@@ -31,6 +31,7 @@ from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
 from ironic.drivers.modules import agent_base
+from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 
@@ -57,6 +58,12 @@ OPTIONAL_PROPERTIES = {
                         'a dot to prefix the domain name. This value will be '
                         'ignored if ``image_http_proxy`` and '
                         '``image_https_proxy`` are not specified. Optional.'),
+    'image_download_source': _('Specifies whether direct deploy interface '
+                               'should try to use the image source directly '
+                               'or if ironic should cache the image on the '
+                               'conductor and serve it from ironic\'s own '
+                               'HTTP server. Accepted values are "swift", '
+                               '"http" and "local". Optional.'),
 }
 
 _RAID_APPLY_CONFIGURATION_ARGSINFO = {
@@ -168,8 +175,19 @@ def validate_http_provisioning_configuration(node):
     :raises: MissingParameterValue if required option(s) is not set.
     """
     image_source = node.instance_info.get('image_source')
-    if (not service_utils.is_glance_image(image_source)
-            or CONF.agent.image_download_source != 'http'):
+    image_download_source = deploy_utils.get_image_download_source(node)
+    if image_download_source not in ('swift', 'http', 'local'):
+        raise exception.InvalidParameterValue(
+            _('Invalid value for image_download_source: "%s". Valid values '
+              'are swift, http or local.') % image_download_source)
+
+    # NOTE(dtantsur): local HTTP configuration is required in two cases:
+    # 1. Glance images with image_download_source == http
+    # 2. File images (since we need to serve them to IPA)
+    if (not image_source.startswith('file://')
+            and image_download_source != 'local'
+            and (not service_utils.is_glance_image(image_source)
+                 or image_download_source == 'swift')):
         return
 
     params = {
@@ -252,6 +270,8 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
                         'step deploy.write_image, using the deprecated '
                         'synchronous fall-back', task.node.uuid)
 
+        client = agent_client.get_client(task)
+
         if self.has_decomposed_deploy_steps and has_write_image:
             configdrive = node.instance_info.get('configdrive')
             # Now switch into the corresponding in-band deploy step and let the
@@ -261,10 +281,10 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
                         'args': {'image_info': image_info,
                                  'configdrive': configdrive}}
             return agent_base.execute_step(task, new_step, 'deploy',
-                                           client=self._client)
+                                           client=client)
         else:
             # TODO(dtantsur): remove in W
-            command = self._client.prepare_image(node, image_info, wait=True)
+            command = client.prepare_image(node, image_info, wait=True)
             if command['command_status'] == 'FAILED':
                 # TODO(jimrollenhagen) power off if using neutron dhcp to
                 #                      align with pxe driver?
@@ -275,25 +295,33 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
 
     # TODO(dtantsur): remove in W
     def _get_uuid_from_result(self, task, type_uuid):
-        command = self._client.get_commands_status(task.node)[-1]
+        client = agent_client.get_client(task)
+        command = client.get_last_command_status(task.node, 'prepare_image')
+        if (not command
+                or not command.get('command_result', {}).get('result')):
+            msg = _('Unexpected response from the agent for node %s: the '
+                    'running command list does not include prepare_image '
+                    'or its result is malformed') % task.node.uuid
+            LOG.error(msg)
+            deploy_utils.set_failed_state(task, msg)
+            return
 
-        if command['command_result'] is not None:
-            words = command['command_result']['result'].split()
-            for word in words:
-                if type_uuid in word:
-                    result = word.split('=')[1]
-                    if not result:
-                        msg = (_('Command result did not return %(type_uuid)s '
-                                 'for node %(node)s. The version of the IPA '
-                                 'ramdisk used in the deployment might not '
-                                 'have support for provisioning of '
-                                 'partition images.') %
-                               {'type_uuid': type_uuid,
-                                'node': task.node.uuid})
-                        LOG.error(msg)
-                        deploy_utils.set_failed_state(task, msg)
-                        return
-                    return result
+        words = command['command_result']['result'].split()
+        for word in words:
+            if type_uuid in word:
+                result = word.split('=')[1]
+                if not result:
+                    msg = (_('Command result did not return %(type_uuid)s '
+                             'for node %(node)s. The version of the IPA '
+                             'ramdisk used in the deployment might not '
+                             'have support for provisioning of '
+                             'partition images.') %
+                           {'type_uuid': type_uuid,
+                            'node': task.node.uuid})
+                    LOG.error(msg)
+                    deploy_utils.set_failed_state(task, msg)
+                    return
+                return result
 
     @METRICS.timer('AgentDeployMixin.prepare_instance_boot')
     @base.deploy_step(priority=60)
@@ -322,8 +350,9 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
         # ppc64* hardware we need to provide the 'PReP_Boot_partition_uuid' to
         # direct where the bootloader should be installed.
         driver_internal_info = task.node.driver_internal_info
+        client = agent_client.get_client(task)
         try:
-            partition_uuids = self._client.get_partition_uuids(node).get(
+            partition_uuids = client.get_partition_uuids(node).get(
                 'command_result') or {}
             root_uuid = partition_uuids.get('root uuid')
         except exception.AgentAPIError:
@@ -379,8 +408,7 @@ class AgentDeployMixin(agent_base.AgentDeployMixin):
             manager_utils.node_set_boot_device(task, 'disk', persistent=True)
 
         # Remove symbolic link when deploy is done.
-        if CONF.agent.image_download_source == 'http':
-            deploy_utils.remove_http_instance_symlink(task.node.uuid)
+        deploy_utils.remove_http_instance_symlink(task.node.uuid)
 
 
 class AgentDeploy(AgentDeployMixin, agent_base.AgentBaseMixin,
@@ -442,7 +470,10 @@ class AgentDeploy(AgentDeployMixin, agent_base.AgentBaseMixin,
 
         deploy_utils.check_for_missing_params(params, error_msg)
 
-        if not service_utils.is_glance_image(image_source):
+        # NOTE(dtantsur): glance images contain a checksum; for file images we
+        # will recalculate the checksum anyway.
+        if (not service_utils.is_glance_image(image_source)
+                and not image_source.startswith('file://')):
 
             def _raise_missing_checksum_exception(node):
                 raise exception.MissingParameterValue(_(
@@ -489,6 +520,9 @@ class AgentDeploy(AgentDeployMixin, agent_base.AgentBaseMixin,
             # immediately to the next deploy step.
             LOG.debug('Performing a fast track deployment for %(node)s.',
                       {'node': task.node.uuid})
+            # NOTE(dtantsur): while the node is up and heartbeating, we don't
+            # necessary have the deploy steps cached. Force a refresh here.
+            self.refresh_steps(task, 'deploy')
         elif task.driver.storage.should_write_image(task):
             # Check if the driver has already performed a reboot in a previous
             # deploy step.
@@ -626,8 +660,7 @@ class AgentDeploy(AgentDeployMixin, agent_base.AgentBaseMixin,
         :param task: a TaskManager instance.
         """
         super(AgentDeploy, self).clean_up(task)
-        if CONF.agent.image_download_source == 'http':
-            deploy_utils.destroy_http_instance_images(task.node)
+        deploy_utils.destroy_http_instance_images(task.node)
 
 
 class AgentRAID(base.RAIDInterface):
