@@ -25,14 +25,17 @@ from oslo_utils import timeutils
 import retrying
 
 from ironic.common import boot_devices
+from ironic.common import dhcp_factory
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import steps as conductor_steps
+from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
+from ironic.drivers import base
 from ironic.drivers.modules import agent_client
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
@@ -69,7 +72,14 @@ VENDOR_PROPERTIES = {
     'deploy_forces_oob_reboot': _(
         'Whether Ironic should force a reboot of the Node via the out-of-band '
         'channel after deployment is complete. Provides compatibility with '
-        'older deploy ramdisks. Defaults to False. Optional.')
+        'older deploy ramdisks. Defaults to False. Optional.'),
+    'agent_verify_ca': _(
+        'Either a Boolean value, a path to a CA_BUNDLE file or directory with '
+        'certificates of trusted CAs. If set to True ironic will verify '
+        'the agent\'s certificate; if False the driver will ignore verifying '
+        'the SSL certificate. If it\'s a path the driver will use the '
+        'specified certificate or one of the certificates in the '
+        'directory. Defaults to True. Optional'),
 }
 
 __HEARTBEAT_RECORD_ONLY = (states.ENROLL, states.MANAGEABLE, states.AVAILABLE,
@@ -89,11 +99,6 @@ _FASTTRACK_HEARTBEAT_ALLOWED = (states.DEPLOYWAIT, states.CLEANWAIT,
 FASTTRACK_HEARTBEAT_ALLOWED = frozenset(_FASTTRACK_HEARTBEAT_ALLOWED)
 
 
-def _get_client():
-    client = agent_client.AgentClient()
-    return client
-
-
 @METRICS.timer('post_clean_step_hook')
 def post_clean_step_hook(interface, step):
     """Decorator method for adding a post clean step hook.
@@ -193,12 +198,14 @@ def _post_step_reboot(task, step_type):
                 'node': task.node.uuid,
                 'err': e,
                 'type': step_type})
-        LOG.error(msg, exc_info=not isinstance(e, exception.IronicException))
+        traceback = not isinstance(e, exception.IronicException)
         # do not set cleaning_reboot if we didn't reboot
         if step_type == 'clean':
-            manager_utils.cleaning_error_handler(task, msg)
+            manager_utils.cleaning_error_handler(task, msg,
+                                                 traceback=traceback)
         else:
-            manager_utils.deploying_error_handler(task, msg)
+            manager_utils.deploying_error_handler(task, msg,
+                                                  traceback=traceback)
         return
 
     # Signify that we've rebooted
@@ -333,6 +340,13 @@ def get_steps(task, step_type, interface=None, override_priorities=None):
     return steps
 
 
+def find_step(task, step_type, interface, name):
+    """Find the given in-band step."""
+    steps = get_steps(task, step_type, interface)
+    return conductor_steps.find_step(
+        steps, {'interface': interface, 'step': name})
+
+
 def _raise(step_type, msg):
     assert step_type in ('clean', 'deploy')
     exc = (exception.NodeCleaningFailure if step_type == 'clean'
@@ -340,18 +354,20 @@ def _raise(step_type, msg):
     raise exc(msg)
 
 
-def execute_step(task, step, step_type):
+def execute_step(task, step, step_type, client=None):
     """Execute a clean or deploy step asynchronously on the agent.
 
     :param task: a TaskManager object containing the node
     :param step: a step dictionary to execute
     :param step_type: 'clean' or 'deploy'
+    :param client: agent client (if available)
     :raises: NodeCleaningFailure (clean step) or InstanceDeployFailure (deploy
         step) if the agent does not return a command status.
     :returns: states.CLEANWAIT/DEPLOYWAIT to signify the step will be
         completed async
     """
-    client = _get_client()
+    if client is None:
+        client = agent_client.get_client(task)
     ports = objects.Port.list_by_node_id(
         task.context, task.node.id)
     call = getattr(client, 'execute_%s_step' % step_type)
@@ -359,8 +375,7 @@ def execute_step(task, step, step_type):
     if not result.get('command_status'):
         _raise(step_type, _(
             'Agent on node %(node)s returned bad command result: '
-            '%(result)s') % {'node': task.node.uuid,
-                             'result': result.get('command_error')})
+            '%(result)s') % {'node': task.node.uuid, 'result': result})
     return states.CLEANWAIT if step_type == 'clean' else states.DEPLOYWAIT
 
 
@@ -369,11 +384,34 @@ def execute_clean_step(task, step):
     return execute_step(task, step, 'clean')
 
 
+def _step_failure_handler(task, msg, step_type, traceback=False):
+    driver_utils.collect_ramdisk_logs(
+        task.node, label='cleaning' if step_type == 'clean' else None)
+    if step_type == 'clean':
+        manager_utils.cleaning_error_handler(task, msg, traceback=traceback)
+    else:
+        manager_utils.deploying_error_handler(task, msg, traceback=traceback)
+
+
 class HeartbeatMixin(object):
     """Mixin class implementing heartbeat processing."""
 
+    has_decomposed_deploy_steps = False
+    """Whether the driver supports decomposed deploy steps.
+
+    Previously (since Rocky), drivers used a single 'deploy' deploy step on
+    the deploy interface. Some additional steps were added for the 'direct'
+    and 'iscsi' deploy interfaces in the Ussuri cycle, which means that
+    more of the deployment flow is driven by deploy steps.
+    """
+
+    collect_deploy_logs = True
+
     def __init__(self):
-        self._client = _get_client()
+        if not self.has_decomposed_deploy_steps:
+            LOG.warning('%s does not support decomposed deploy steps. This '
+                        'is deprecated and will stop working in a future '
+                        'release', self.__class__.__name__)
 
     def continue_deploy(self, task):
         """Continues the deployment of baremetal node.
@@ -455,10 +493,11 @@ class HeartbeatMixin(object):
         node = task.node
         if (node.provision_state in (states.CLEANING, states.CLEANWAIT)
                 and not CONF.conductor.allow_provisioning_in_maintenance):
-            LOG.error('Aborting cleaning for node %s, as it is in maintenance '
-                      'mode', node.uuid)
+            log_msg = ('Aborting cleaning for node %s, as it is in '
+                       'maintenance mode' % node.uuid)
             last_error = _('Cleaning aborted as node is in maintenance mode')
-            manager_utils.cleaning_error_handler(task, last_error)
+            manager_utils.cleaning_error_handler(task, log_msg,
+                                                 errmsg=last_error)
         elif (node.provision_state in (states.DEPLOYING, states.DEPLOYWAIT)
               and not CONF.conductor.allow_provisioning_in_maintenance):
             LOG.error('Aborting deployment for node %s, as it is in '
@@ -491,8 +530,12 @@ class HeartbeatMixin(object):
             # are currently in the core deploy.deploy step. Other deploy steps
             # may cause the agent to boot, but we should not trigger deployment
             # at that point if the driver is polling for completion of a step.
-            if self.in_core_deploy_step(task):
+            if (not self.has_decomposed_deploy_steps
+                    and self.in_core_deploy_step(task)):
                 msg = _('Failed checking if deploy is done')
+                # NOTE(mgoddard): support backwards compatibility for
+                # drivers which do not implement continue_deploy and
+                # reboot_to_instance as deploy steps.
                 if not self.deploy_has_started(task):
                     msg = _('Node failed to deploy')
                     self.continue_deploy(task)
@@ -517,7 +560,7 @@ class HeartbeatMixin(object):
             # Do not call the error handler is the node is already DEPLOYFAIL
             if node.provision_state in (states.DEPLOYING, states.DEPLOYWAIT):
                 deploy_utils.set_failed_state(
-                    task, last_error, collect_logs=bool(self._client))
+                    task, last_error, collect_logs=self.collect_deploy_logs)
 
     def _heartbeat_clean_wait(self, task):
         node = task.node
@@ -545,10 +588,11 @@ class HeartbeatMixin(object):
                     self.continue_cleaning(task)
         except Exception as e:
             last_error = _('%(msg)s. Error: %(exc)s') % {'msg': msg, 'exc': e}
-            LOG.exception('Asynchronous exception for node %(node)s: %(err)s',
-                          {'node': task.node.uuid, 'err': last_error})
+            log_msg = ('Asynchronous exception for node %(node)s: %(err)s' %
+                       {'node': task.node.uuid, 'err': last_error})
             if node.provision_state in (states.CLEANING, states.CLEANWAIT):
-                manager_utils.cleaning_error_handler(task, last_error)
+                manager_utils.cleaning_error_handler(task, log_msg,
+                                                     errmsg=last_error)
 
     def _heartbeat_rescue_wait(self, task):
         msg = _('Node failed to perform rescue operation')
@@ -563,12 +607,14 @@ class HeartbeatMixin(object):
                 manager_utils.rescuing_error_handler(task, last_error)
 
     @METRICS.timer('HeartbeatMixin.heartbeat')
-    def heartbeat(self, task, callback_url, agent_version):
+    def heartbeat(self, task, callback_url, agent_version,
+                  agent_verify_ca=None):
         """Process a heartbeat.
 
         :param task: task to work with.
         :param callback_url: agent HTTP API URL.
         :param agent_version: The version of the agent that is heartbeating
+        :param agent_verify_ca: TLS certificate for the agent.
         """
         # NOTE(pas-ha) immediately skip the rest if nothing to do
         if (task.node.provision_state not in self.heartbeat_allowed_states
@@ -580,7 +626,7 @@ class HeartbeatMixin(object):
             return
 
         try:
-            task.upgrade_lock()
+            task.upgrade_lock(retry=False)
         except exception.NodeLocked:
             LOG.warning('Node %s is currently locked, skipping heartbeat '
                         'processing (will retry on the next heartbeat)',
@@ -597,6 +643,8 @@ class HeartbeatMixin(object):
         # datetime.datetime.strptime(var, "%Y-%m-%d %H:%M:%S.%f")
         driver_internal_info['agent_last_heartbeat'] = str(
             timeutils.utcnow().isoformat())
+        if agent_verify_ca:
+            driver_internal_info['agent_verify_ca'] = agent_verify_ca
         node.driver_internal_info = driver_internal_info
         node.save()
 
@@ -626,7 +674,8 @@ class HeartbeatMixin(object):
         """
         node = task.node
         try:
-            result = self._client.finalize_rescue(node)
+            client = agent_client.get_client(task)
+            result = client.finalize_rescue(node)
         except exception.IronicException as e:
             raise exception.InstanceRescueFailure(node=node.uuid,
                                                   instance=node.instance_uuid,
@@ -637,7 +686,7 @@ class HeartbeatMixin(object):
             # handler.
             fail_reason = (_('Agent returned bad result for command '
                              'finalize_rescue: %(result)s') %
-                           {'result': result.get('command_error')})
+                           {'result': agent_client.get_command_error(result)})
             raise exception.InstanceRescueFailure(node=node.uuid,
                                                   instance=node.instance_uuid,
                                                   reason=fail_reason)
@@ -648,7 +697,142 @@ class HeartbeatMixin(object):
         task.process_event('done')
 
 
-class AgentDeployMixin(HeartbeatMixin):
+class AgentBaseMixin(object):
+    """Mixin with base methods not relying on any deploy steps."""
+
+    def should_manage_boot(self, task):
+        """Whether agent boot is managed by ironic."""
+        return True
+
+    @METRICS.timer('AgentBaseMixin.tear_down')
+    @task_manager.require_exclusive_lock
+    def tear_down(self, task):
+        """Tear down a previous deployment on the task's node.
+
+        Power off the node. All actual clean-up is done in the clean_up()
+        method which should be called separately.
+
+        :param task: a TaskManager instance containing the node to act on.
+        :returns: deploy state DELETED.
+        :raises: NetworkError if the cleaning ports cannot be removed.
+        :raises: InvalidParameterValue when the wrong state is specified
+             or the wrong driver info is specified.
+        :raises: StorageError when volume detachment fails.
+        :raises: other exceptions by the node's power driver if something
+             wrong occurred during the power action.
+        """
+        manager_utils.node_power_action(task, states.POWER_OFF)
+        task.driver.storage.detach_volumes(task)
+        deploy_utils.tear_down_storage_configuration(task)
+        with manager_utils.power_state_for_network_configuration(task):
+            task.driver.network.unconfigure_tenant_networks(task)
+            # NOTE(mgoddard): If the deployment was unsuccessful the node may
+            # have ports on the provisioning network which were not deleted.
+            task.driver.network.remove_provisioning_network(task)
+        return states.DELETED
+
+    @METRICS.timer('AgentBaseMixin.clean_up')
+    def clean_up(self, task):
+        """Clean up the deployment environment for the task's node.
+
+        Unlinks TFTP and instance images and triggers image cache cleanup.
+        Removes the TFTP configuration files for this node.
+
+        :param task: a TaskManager instance containing the node to act on.
+        """
+        if self.should_manage_boot(task):
+            task.driver.boot.clean_up_ramdisk(task)
+        task.driver.boot.clean_up_instance(task)
+        provider = dhcp_factory.DHCPFactory()
+        provider.clean_dhcp(task)
+
+    def take_over(self, task):
+        """Take over management of this node from a dead conductor.
+
+        :param task: a TaskManager instance.
+        """
+        pass
+
+    @METRICS.timer('AgentDeployMixin.prepare_cleaning')
+    def prepare_cleaning(self, task):
+        """Boot into the agent to prepare for cleaning.
+
+        :param task: a TaskManager object containing the node
+        :raises: NodeCleaningFailure, NetworkError if the previous cleaning
+            ports cannot be removed or if new cleaning ports cannot be created.
+        :raises: InvalidParameterValue if cleaning network UUID config option
+            has an invalid value.
+        :returns: states.CLEANWAIT to signify an asynchronous prepare
+        """
+        return deploy_utils.prepare_inband_cleaning(
+            task, manage_boot=self.should_manage_boot(task))
+
+    @METRICS.timer('AgentDeployMixin.tear_down_cleaning')
+    def tear_down_cleaning(self, task):
+        """Clean up the PXE and DHCP files after cleaning.
+
+        :param task: a TaskManager object containing the node
+        :raises: NodeCleaningFailure, NetworkError if the cleaning ports cannot
+            be removed
+        """
+        deploy_utils.tear_down_inband_cleaning(
+            task, manage_boot=self.should_manage_boot(task))
+
+
+class AgentOobStepsMixin(object):
+    """Mixin with out-of-band deploy steps."""
+
+    @METRICS.timer('AgentDeployMixin.switch_to_tenant_network')
+    @base.deploy_step(priority=30)
+    @task_manager.require_exclusive_lock
+    def switch_to_tenant_network(self, task):
+        """Deploy step to switch the node to the tenant network.
+
+        :param task: a TaskManager object containing the node
+        """
+        try:
+            with manager_utils.power_state_for_network_configuration(task):
+                task.driver.network.remove_provisioning_network(task)
+                task.driver.network.configure_tenant_networks(task)
+        except Exception as e:
+            msg = (_('Error changing node %(node)s to tenant networks after '
+                     'deploy. %(cls)s: %(error)s') %
+                   {'node': task.node.uuid, 'cls': e.__class__.__name__,
+                    'error': e})
+            # NOTE(mgoddard): Don't collect logs since the node has been
+            # powered off.
+            log_and_raise_deployment_error(task, msg, collect_logs=False,
+                                           exc=e)
+
+    @METRICS.timer('AgentDeployMixin.boot_instance')
+    @base.deploy_step(priority=20)
+    @task_manager.require_exclusive_lock
+    def boot_instance(self, task):
+        """Deploy step to boot the final instance.
+
+        :param task: a TaskManager object containing the node
+        """
+        can_power_on = (states.POWER_ON in
+                        task.driver.power.get_supported_power_states(task))
+        try:
+            if can_power_on:
+                manager_utils.node_power_action(task, states.POWER_ON)
+            else:
+                LOG.debug('Not trying to power on node %s that does not '
+                          'support powering on, assuming already running',
+                          task.node.uuid)
+        except Exception as e:
+            msg = (_('Error booting node %(node)s after deploy. '
+                     '%(cls)s: %(error)s') %
+                   {'node': task.node.uuid, 'cls': e.__class__.__name__,
+                    'error': e})
+            # NOTE(mgoddard): Don't collect logs since the node has been
+            # powered off.
+            log_and_raise_deployment_error(task, msg, collect_logs=False,
+                                           exc=e)
+
+
+class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
     """Mixin with deploy methods."""
 
     @METRICS.timer('AgentDeployMixin.get_clean_steps')
@@ -712,11 +896,17 @@ class AgentDeployMixin(HeartbeatMixin):
                   {'node': node.uuid, 'type': step_type,
                    'steps': previous_steps})
 
-        call = getattr(self._client, 'get_%s_steps' % step_type)
-        # TODO(dtantsur): remove the error handling in the V release.
+        client = agent_client.get_client(task)
+        call = getattr(client, 'get_%s_steps' % step_type)
         try:
             agent_result = call(node, task.ports).get('command_result', {})
-        except exception.AgentAPIError as exc:
+        except exception.AgentInProgress as exc:
+            LOG.debug('Agent for node %(node)s is busy with a command, '
+                      'will refresh steps on the next heartbeat.',
+                      {'node': task.node.uuid})
+            return
+
+            # TODO(dtantsur): change to just 'raise'
             if step_type == 'clean':
                 raise
             else:
@@ -823,14 +1013,14 @@ class AgentDeployMixin(HeartbeatMixin):
             msg = (_('Could not continue cleaning on node '
                      '%(node)s: %(err)s.') %
                    {'node': node.uuid, 'err': e})
-            LOG.exception(msg)
-            return manager_utils.cleaning_error_handler(task, msg)
+            return manager_utils.cleaning_error_handler(task, msg,
+                                                        traceback=True)
         except exception.InstanceDeployFailure as e:
             msg = (_('Could not continue deployment on node '
                      '%(node)s: %(err)s.') %
                    {'node': node.uuid, 'err': e})
-            LOG.exception(msg)
-            return manager_utils.deploying_error_handler(task, msg)
+            return manager_utils.deploying_error_handler(task, msg,
+                                                         traceback=True)
 
         if manual_clean:
             # Don't restart manual cleaning if agent reboots to a new
@@ -859,15 +1049,15 @@ class AgentDeployMixin(HeartbeatMixin):
                          '%(node)s after step %(step)s: %(err)s.') %
                        {'node': node.uuid, 'err': e,
                         'step': node.clean_step})
-                LOG.exception(msg)
-                return manager_utils.cleaning_error_handler(task, msg)
+                return manager_utils.cleaning_error_handler(task, msg,
+                                                            traceback=True)
             except exception.InstanceDeployFailure as e:
                 msg = (_('Could not restart deployment on node '
                          '%(node)s after step %(step)s: %(err)s.') %
                        {'node': node.uuid, 'err': e,
                         'step': node.deploy_step})
-                LOG.exception(msg)
-                return manager_utils.deploying_error_handler(task, msg)
+                return manager_utils.deploying_error_handler(task, msg,
+                                                             traceback=True)
 
         manager_utils.notify_conductor_resume_operation(task, step_type)
 
@@ -891,7 +1081,8 @@ class AgentDeployMixin(HeartbeatMixin):
         assert step_type in ('clean', 'deploy')
 
         node = task.node
-        agent_commands = self._client.get_commands_status(task.node)
+        client = agent_client.get_client(task)
+        agent_commands = client.get_commands_status(task.node)
 
         if _freshly_booted(agent_commands, step_type):
             field = ('cleaning_reboot' if step_type == 'clean'
@@ -918,11 +1109,10 @@ class AgentDeployMixin(HeartbeatMixin):
             msg = (_('Agent returned error for %(type)s step %(step)s on node '
                      '%(node)s : %(err)s.') %
                    {'node': node.uuid,
-                    'err': command.get('command_error'),
+                    'err': agent_client.get_command_error(command),
                     'step': current_step,
                     'type': step_type})
-            LOG.error(msg)
-            return manager_utils.cleaning_error_handler(task, msg)
+            return _step_failure_handler(task, msg, step_type)
         # NOTE(dtantsur): VERSION_MISMATCH is a new alias for
         # CLEAN_VERSION_MISMATCH, remove the old one after IPA removes it.
         elif command.get('command_status') in ('CLEAN_VERSION_MISMATCH',
@@ -949,11 +1139,8 @@ class AgentDeployMixin(HeartbeatMixin):
                             'cls': e.__class__.__name__,
                             'step': current_step,
                             'type': step_type})
-                    LOG.exception(msg)
-                    if step_type == 'clean':
-                        return manager_utils.cleaning_error_handler(task, msg)
-                    else:
-                        return manager_utils.deploying_error_handler(task, msg)
+                    return _step_failure_handler(task, msg, step_type,
+                                                 traceback=True)
 
             if current_step.get('reboot_requested'):
                 _post_step_reboot(task, step_type)
@@ -970,22 +1157,15 @@ class AgentDeployMixin(HeartbeatMixin):
                     'err': command.get('command_status'),
                     'step': current_step,
                     'type': step_type})
-            LOG.error(msg)
-            if step_type == 'clean':
-                return manager_utils.cleaning_error_handler(task, msg)
-            else:
-                return manager_utils.deploying_error_handler(task, msg)
+            return _step_failure_handler(task, msg, step_type)
 
-    @METRICS.timer('AgentDeployMixin.reboot_and_finish_deploy')
-    def reboot_and_finish_deploy(self, task):
-        """Helper method to trigger reboot on the node and finish deploy.
-
-        This method initiates a reboot on the node. On success, it
-        marks the deploy as complete. On failure, it logs the error
-        and marks deploy as failure.
+    @METRICS.timer('AgentDeployMixin.tear_down_agent')
+    @base.deploy_step(priority=40)
+    @task_manager.require_exclusive_lock
+    def tear_down_agent(self, task):
+        """A deploy step to tear down the agent.
 
         :param task: a TaskManager object containing the node
-        :raises: InstanceDeployFailure, if node reboot failed.
         """
         wait = CONF.agent.post_deploy_get_power_state_retry_interval * 1000
         attempts = CONF.agent.post_deploy_get_power_state_retries + 1
@@ -1007,25 +1187,43 @@ class AgentDeployMixin(HeartbeatMixin):
         # in-band methods
         oob_power_off = strutils.bool_from_string(
             node.driver_info.get('deploy_forces_oob_reboot', False))
+        can_power_on = (states.POWER_ON in
+                        task.driver.power.get_supported_power_states(task))
 
+        client = agent_client.get_client(task)
         try:
-            if not oob_power_off:
+            if not can_power_on:
+                LOG.info('Power interface of node %(node)s does not support '
+                         'power on, using reboot to switch to the instance',
+                         node.uuid)
+                client.sync(node)
+                manager_utils.node_power_action(task, states.REBOOT)
+            elif not oob_power_off:
                 try:
-                    self._client.power_off(node)
-                    _wait_until_powered_off(task)
+                    client.power_off(node)
                 except Exception as e:
-                    LOG.warning('Failed to soft power off node %(node_uuid)s '
-                                'in at least %(timeout)d seconds. '
+                    LOG.warning('Failed to soft power off node %(node_uuid)s. '
                                 '%(cls)s: %(error)s',
                                 {'node_uuid': node.uuid,
-                                 'timeout': (wait * (attempts - 1)) / 1000,
                                  'cls': e.__class__.__name__, 'error': e},
                                 exc_info=not isinstance(
                                     e, exception.IronicException))
+
+                # NOTE(dtantsur): in rare cases it may happen that the power
+                # off request comes through but we never receive the response.
+                # Check the power state before trying to force off.
+                try:
+                    _wait_until_powered_off(task)
+                except Exception:
+                    LOG.warning('Failed to soft power off node %(node_uuid)s '
+                                'in at least %(timeout)d seconds. Forcing '
+                                'hard power off and proceeding.',
+                                {'node_uuid': node.uuid,
+                                 'timeout': (wait * (attempts - 1)) / 1000})
                     manager_utils.node_power_action(task, states.POWER_OFF)
             else:
                 # Flush the file system prior to hard rebooting the node
-                result = self._client.sync(node)
+                result = client.sync(node)
                 error = result.get('faultstring')
                 if error:
                     if 'Unknown command' in error:
@@ -1045,23 +1243,21 @@ class AgentDeployMixin(HeartbeatMixin):
                     'error': e})
             log_and_raise_deployment_error(task, msg, exc=e)
 
-        try:
-            with manager_utils.power_state_for_network_configuration(task):
-                task.driver.network.remove_provisioning_network(task)
-                task.driver.network.configure_tenant_networks(task)
-            manager_utils.node_power_action(task, states.POWER_ON)
-        except Exception as e:
-            msg = (_('Error rebooting node %(node)s after deploy. '
-                     '%(cls)s: %(error)s') %
-                   {'node': node.uuid, 'cls': e.__class__.__name__,
-                    'error': e})
-            # NOTE(mgoddard): Don't collect logs since the node has been
-            # powered off.
-            log_and_raise_deployment_error(task, msg, collect_logs=False,
-                                           exc=e)
+    # TODO(dtantsur): remove in W
+    @METRICS.timer('AgentDeployMixin.reboot_and_finish_deploy')
+    def reboot_and_finish_deploy(self, task):
+        """Helper method to trigger reboot on the node and finish deploy.
 
-        # TODO(dtantsur): remove these two calls when this function becomes a
-        # real deploy step.
+        This method initiates a reboot on the node. On success, it
+        marks the deploy as complete. On failure, it logs the error
+        and marks deploy as failure.
+
+        :param task: a TaskManager object containing the node
+        :raises: InstanceDeployFailure, if node reboot failed.
+        """
+        # NOTE(dtantsur): do nothing here, the new deploy steps tear_down_agent
+        # and boot_instance will be picked up and finish the deploy (even for
+        # legacy deploy interfaces without decomposed steps).
         task.process_event('wait')
         manager_utils.notify_conductor_resume_deploy(task)
 
@@ -1170,7 +1366,8 @@ class AgentDeployMixin(HeartbeatMixin):
                       'partition %(part)s, EFI system partition %(efi)s',
                       {'node': node.uuid, 'part': root_uuid,
                        'efi': efi_system_part_uuid})
-            result = self._client.install_bootloader(
+            client = agent_client.get_client(task)
+            result = client.install_bootloader(
                 node, root_uuid=root_uuid,
                 efi_system_part_uuid=efi_system_part_uuid,
                 prep_boot_part_uuid=prep_boot_part_uuid,
@@ -1182,7 +1379,7 @@ class AgentDeployMixin(HeartbeatMixin):
                     msg = (_("Failed to install a bootloader when "
                              "deploying node %(node)s. Error: %(error)s") %
                            {'node': node.uuid,
-                            'error': result['command_error']})
+                            'error': agent_client.get_command_error(result)})
                     log_and_raise_deployment_error(task, msg)
                 else:
                     # Its possible the install will fail if the IPA image
@@ -1190,7 +1387,7 @@ class AgentDeployMixin(HeartbeatMixin):
                     LOG.info('Could not install bootloader for whole disk '
                              'image for node %(node)s, Error: %(error)s"',
                              {'node': node.uuid,
-                              'error': result['command_error']})
+                              'error': agent_client.get_command_error(result)})
                     return
 
         try:
