@@ -13,10 +13,12 @@
 # limitations under the License.
 
 from http import client as http_client
+import os
 
 from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_serialization import jsonutils
+from oslo_utils import strutils
 import requests
 import retrying
 
@@ -33,6 +35,15 @@ DEFAULT_IPA_PORTAL_PORT = 3260
 REBOOT_COMMAND = 'run_image'
 
 
+def get_client(task):
+    """Get client for this node."""
+    try:
+        return task.cached_agent_client
+    except AttributeError:
+        task.cached_agent_client = AgentClient()
+        return task.cached_agent_client
+
+
 def get_command_error(command):
     """Extract an error string from the command result.
 
@@ -77,7 +88,18 @@ class AgentClient(object):
         })
 
     def _get_verify(self, node):
-        return node.driver_info.get('agent_verify_ca', True)
+        value = (node.driver_internal_info.get('agent_verify_ca')
+                 or node.driver_info.get('agent_verify_ca')
+                 or CONF.agent.verify_ca)
+        if isinstance(value, str):
+            try:
+                value = strutils.bool_from_string(value, strict=True)
+            except ValueError:
+                if not os.path.exists(value):
+                    raise exception.InvalidParameterValue(
+                        _('Agent CA %s is neither a path nor a boolean')
+                        % value)
+        return value
 
     def _raise_if_typeerror(self, result, node, method):
         error = result.get('command_error')
@@ -102,23 +124,11 @@ class AgentClient(object):
         :param method: A string represents the command executed by agent.
         :raises: AgentCommandTimeout if timeout is reached.
         """
-        try:
-            method = method.split('.', 1)[1]
-        except IndexError:
-            pass
-
         # NOTE(dtantsur): this function uses AgentCommandTimeout on every
         # failure, but unless the timeout is reached, the exception is caught
         # and retried by the @retry decorator above.
-
-        commands = self.get_commands_status(node)
-        try:
-            result = next(c for c in reversed(commands)
-                          if c.get('command_name') == method)
-        except StopIteration:
-            LOG.debug('Command %(cmd)s is not in the executing commands list '
-                      'for node %(node)s',
-                      {'cmd': method, 'node': node.uuid})
+        result = self.get_last_command_status(node, method)
+        if result is None:
             raise exception.AgentCommandTimeout(command=method, node=node.uuid)
 
         if result.get('command_status') == 'RUNNING':
@@ -152,6 +162,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command result from agent, see
                   get_commands_status for a sample.
         """
@@ -174,11 +186,9 @@ class AgentClient(object):
                 verify=self._get_verify(node),
                 timeout=CONF.agent.command_timeout)
         except (requests.ConnectionError, requests.Timeout) as e:
-            msg = (_('Failed to connect to the agent running on node %(node)s '
-                     'for invoking command %(method)s. Error: %(error)s') %
-                   {'node': node.uuid, 'method': method, 'error': e})
-            LOG.error(msg)
-            raise exception.AgentConnectionFailed(reason=msg)
+            result = self._handle_timeout_on_command_execution(node, method,
+                                                               params, e)
+            response = None
         except requests.RequestException as e:
             msg = (_('Error invoking agent command %(method)s for node '
                      '%(node)s. Error: %(error)s') %
@@ -186,36 +196,59 @@ class AgentClient(object):
             LOG.error(msg)
             raise exception.IronicException(msg)
 
-        # TODO(russellhaering): real error handling
-        try:
-            result = response.json()
-        except ValueError:
-            msg = _(
-                'Unable to decode response as JSON.\n'
-                'Request URL: %(url)s\nRequest body: "%(body)s"\n'
-                'Response status code: %(code)s\n'
-                'Response: "%(response)s"'
-            ) % ({'response': response.text, 'body': body, 'url': url,
-                  'code': response.status_code})
-            LOG.error(msg)
-            raise exception.IronicException(msg)
+        if response is not None:
+            # TODO(russellhaering): real error handling
+            try:
+                result = response.json()
+            except ValueError:
+                msg = _(
+                    'Unable to decode response as JSON.\n'
+                    'Request URL: %(url)s\nRequest body: "%(body)s"\n'
+                    'Response status code: %(code)s\n'
+                    'Response: "%(response)s"'
+                ) % ({'response': response.text, 'body': body, 'url': url,
+                      'code': response.status_code})
+                LOG.error(msg)
+                raise exception.IronicException(msg)
 
         error = result.get('command_error')
         LOG.debug('Agent command %(method)s for node %(node)s returned '
-                  'result %(res)s, error %(error)s, HTTP status code %(code)d',
+                  'result %(res)s, error %(error)s, HTTP status code %(code)s',
                   {'node': node.uuid, 'method': method,
                    'res': result.get('command_result'),
                    'error': error,
-                   'code': response.status_code})
-
-        if response.status_code >= http_client.BAD_REQUEST:
-            LOG.error('Agent command %(method)s for node %(node)s failed. '
-                      'Expected 2xx HTTP status code, got %(code)d.',
-                      {'method': method, 'node': node.uuid,
-                       'code': response.status_code})
+                   'code': response.status_code if response is not None
+                   else 'unknown'})
+        if (response is not None
+                and response.status_code >= http_client.BAD_REQUEST):
+            faultstring = result.get('faultstring')
+            if 'agent_token' in faultstring:
+                LOG.error('Agent command %(method)s for node %(node)s '
+                          'failed. Expected 2xx HTTP status code, got '
+                          '%(code)d. Error suggests an older ramdisk '
+                          'which does not support ``agent_token``. '
+                          'This is a fatal error.',
+                          {'method': method, 'node': node.uuid,
+                           'code': response.status_code})
+            else:
+                LOG.error('Agent command %(method)s for node %(node)s failed. '
+                          'Expected 2xx HTTP status code, got %(code)d.',
+                          {'method': method, 'node': node.uuid,
+                           'code': response.status_code})
+            if (response.status_code == http_client.CONFLICT
+                or 'agent is busy' in faultstring.lower()):
+                # HTTP 409 check as an explicit check of if the agent
+                # is already busy.
+                # NOTE(TheJulia): The agent sends upper case A as of
+                # late victoria, but lower case the entire message
+                # for compatability with pre-late victoria agents
+                # which returns HTTP 409.
+                raise exception.AgentInProgress(node=node.uuid,
+                                                command=method,
+                                                error=faultstring)
             raise exception.AgentAPIError(node=node.uuid,
                                           status=response.status_code,
-                                          error=result.get('faultstring'))
+                                          error=faultstring)
 
         self._raise_if_typeerror(result, node, method)
 
@@ -292,6 +325,84 @@ class AgentClient(object):
                   {'node': node.uuid, 'status': status})
         return result
 
+    def _status_if_last_command_matches(self, node, method, params):
+        """Return the status of the given command if it's the last running."""
+        try:
+            method = method.split('.', 1)[1]
+        except IndexError:
+            pass
+
+        commands = self.get_commands_status(node)
+        if not commands:
+            return None
+
+        # TODO(dtantsur): a more reliable way to detect repeated execution
+        # would be to pass a sort of require ID to the agent.
+
+        command = commands[-1]
+        if command['command_name'] != method:
+            LOG.debug('Command %(cmd)s is not currently executing, the last '
+                      'command is %(curr)s',
+                      {'cmd': method, 'curr': command['command_name']})
+            return None
+
+        if command['command_status'] != 'RUNNING':
+            LOG.debug('Command %(cmd)s is not currently executing, its status '
+                      'is %(curr)s',
+                      {'cmd': method, 'curr': command['command_status']})
+            return None
+
+        return command
+
+    def _handle_timeout_on_command_execution(self, node, method, params,
+                                             error):
+        result = None
+        # NOTE(dtantsur): it is possible, especially with eventlet+TLS, that
+        # agent receives a command but fails to return the result to Ironic.
+        # To avoid a failure, check if the last command is the one we're trying
+        # to execute.
+        try:
+            result = self._status_if_last_command_matches(node, method, params)
+        except Exception as e:
+            msg = (_('Failed to connect to the agent running on node '
+                     '%(node)s for checking the last command status '
+                     'after failing to invoke command %(method)s. '
+                     'Error: %(error)s') %
+                   {'node': node.uuid, 'method': method, 'error': e})
+            LOG.error(msg)
+
+        if result is None:
+            msg = (_('Failed to connect to the agent running on node %(node)s '
+                     'for invoking command %(method)s. Error: %(error)s') %
+                   {'node': node.uuid, 'method': method, 'error': error})
+            LOG.error(msg)
+            raise exception.AgentConnectionFailed(reason=msg)
+
+        return result
+
+    def get_last_command_status(self, node, method):
+        """Get the last status for the given command.
+
+        :param node: A Node object.
+        :param method: Command name.
+        :returns: A dict containing command status from agent or None
+            if the command was not found.
+        """
+        try:
+            method = method.split('.', 1)[1]
+        except IndexError:
+            pass
+
+        commands = self.get_commands_status(node)
+        try:
+            return next(c for c in reversed(commands)
+                        if c.get('command_name') == method)
+        except StopIteration:
+            LOG.debug('Command %(cmd)s is not in the executing commands list '
+                      'for node %(node)s',
+                      {'cmd': method, 'node': node.uuid})
+            return None
+
     @METRICS.timer('AgentClient.prepare_image')
     def prepare_image(self, node, image_info, wait=False):
         """Call the `prepare_image` method on the node.
@@ -304,6 +415,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command status from agent.
                   See :func:`get_commands_status` for a command result sample.
         """
@@ -337,6 +450,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
                   See :func:`get_commands_status` for a command result sample.
         """
@@ -367,6 +482,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
                   See :func:`get_commands_status` for a command result sample.
         """
@@ -423,6 +540,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
             See :func:`get_commands_status` for a command result sample.
             The value of key command_result is in the form of:
@@ -454,6 +573,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
             See :func:`get_commands_status` for a command result sample.
             The value of key command_result is in the form of:
@@ -517,6 +638,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
             See :func:`get_commands_status` for a command result sample.
             The value of key command_result is in the form of:
@@ -548,6 +671,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
 
         """
@@ -564,6 +689,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
                   See :func:`get_commands_status` for a command result sample.
         """
@@ -579,6 +706,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
                   See :func:`get_commands_status` for a command result sample.
         """
@@ -594,6 +723,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
                   See :func:`get_commands_status` for a command result sample.
         """
@@ -610,6 +741,8 @@ class AgentClient(object):
         :raises: IronicException when failed to issue the request or there was
                  a malformed response from the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :returns: A dict containing command response from agent.
                   See :func:`get_commands_status` for a command result sample.
         """
@@ -627,6 +760,8 @@ class AgentClient(object):
                  to issue the request, or there was a malformed response from
                  the agent.
         :raises: AgentAPIError when agent failed to execute specified command.
+        :raises: AgentInProgress when the command fails to execute as the agent
+                 is presently executing the prior command.
         :raises: InstanceRescueFailure when the agent ramdisk is too old
                  to support transmission of the rescue password.
         :returns: A dict containing command response from agent.
