@@ -161,6 +161,8 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertEqual(
                 fast_track,
                 bool(task.node.driver_internal_info.get('agent_secret_token')))
+            self.assertEqual(self.service.conductor.id,
+                             task.node.conductor_affinity)
 
     def test__do_node_deploy_ok(self):
         self._test__do_node_deploy_ok()
@@ -246,7 +248,7 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertRaises(exception.SwiftOperationError,
                           deployments.do_node_deploy, task,
                           self.service.conductor.id,
-                          configdrive=b'fake config drive')
+                          configdrive='fake config drive')
         node.refresh()
         self.assertEqual(states.DEPLOYFAIL, node.provision_state)
         self.assertEqual(states.ACTIVE, node.target_provision_state)
@@ -271,8 +273,8 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertRaises(db_exception.DBDataError,
                               deployments.do_node_deploy, task,
                               self.service.conductor.id,
-                              configdrive=b'fake config drive')
-            expected_instance_info.update(configdrive=b'fake config drive')
+                              configdrive='fake config drive')
+            expected_instance_info.update(configdrive='fake config drive')
             expected_calls = [
                 mock.call(node.uuid,
                           {'version': mock.ANY,
@@ -308,7 +310,7 @@ class DoNodeDeployTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertRaises(RuntimeError,
                           deployments.do_node_deploy, task,
                           self.service.conductor.id,
-                          configdrive=b'fake config drive')
+                          configdrive='fake config drive')
         node.refresh()
         self.assertEqual(states.DEPLOYFAIL, node.provision_state)
         self.assertEqual(states.ACTIVE, node.target_provision_state)
@@ -434,7 +436,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
 
-        deployments.do_next_deploy_step(task, None, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, None)
 
         node.refresh()
         self.assertEqual(states.ACTIVE, node.provision_state)
@@ -455,14 +457,13 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
 
-        deployments.do_next_deploy_step(task, 0, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, 0)
 
         node.refresh()
         self.assertEqual(states.DEPLOYWAIT, node.provision_state)
         self.assertEqual(states.ACTIVE, node.target_provision_state)
         self.assertEqual(expected_first_step, node.deploy_step)
         self.assertEqual(0, node.driver_internal_info['deploy_step_index'])
-        self.assertEqual(self.service.conductor.id, node.conductor_affinity)
         mock_execute.assert_called_once_with(mock.ANY, task,
                                              self.deploy_steps[0])
 
@@ -487,7 +488,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
 
-        deployments.do_next_deploy_step(task, 0, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, 0)
 
         node.refresh()
         self.assertIsNone(node.last_error)
@@ -495,7 +496,6 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         self.assertEqual(states.ACTIVE, node.target_provision_state)
         self.assertEqual(expected_first_step, node.deploy_step)
         self.assertEqual(0, node.driver_internal_info['deploy_step_index'])
-        self.assertEqual(self.service.conductor.id, node.conductor_affinity)
         mock_execute.assert_called_once_with(mock.ANY, task,
                                              self.deploy_steps[0])
 
@@ -517,7 +517,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('resume')
 
-        deployments.do_next_deploy_step(task, 1, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, 1)
         node.refresh()
 
         self.assertEqual(states.DEPLOYWAIT, node.provision_state)
@@ -553,7 +553,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('resume')
 
-        deployments.do_next_deploy_step(task, None, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, None)
         node.refresh()
         # Deploying should be complete without calling additional steps
         self.assertEqual(states.ACTIVE, node.provision_state)
@@ -595,7 +595,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
 
-        deployments.do_next_deploy_step(task, 1, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, 0)
 
         # Deploying should be complete
         node.refresh()
@@ -604,8 +604,9 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         self.assertEqual({}, node.deploy_step)
         self.assertNotIn('deploy_step_index', node.driver_internal_info)
         self.assertIsNone(node.driver_internal_info['deploy_steps'])
-        mock_execute.assert_has_calls = [mock.call(self.deploy_steps[0]),
-                                         mock.call(self.deploy_steps[1])]
+        mock_execute.assert_has_calls(
+            [mock.call(task.driver.deploy, task, self.deploy_steps[0]),
+             mock.call(task.driver.deploy, task, self.deploy_steps[1])])
         self.assertNotIn('agent_url', node.driver_internal_info)
         self.assertNotIn('agent_secret_token', node.driver_internal_info)
 
@@ -628,7 +629,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
 
-        deployments.do_next_deploy_step(task, 1, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, 0)
 
         # Deploying should be complete
         node.refresh()
@@ -637,8 +638,9 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         self.assertEqual({}, node.deploy_step)
         self.assertNotIn('deploy_step_index', node.driver_internal_info)
         self.assertIsNone(node.driver_internal_info['deploy_steps'])
-        mock_execute.assert_has_calls = [mock.call(self.deploy_steps[0]),
-                                         mock.call(self.deploy_steps[1])]
+        mock_execute.assert_has_calls(
+            [mock.call(task.driver.deploy, task, self.deploy_steps[0]),
+             mock.call(task.driver.deploy, task, self.deploy_steps[1])])
         self.assertEqual('url', node.driver_internal_info['agent_url'])
         self.assertEqual('token',
                          node.driver_internal_info['agent_secret_token'])
@@ -661,7 +663,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
 
-        deployments.do_next_deploy_step(task, 0, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, 0)
 
         # Make sure we go to DEPLOYFAIL, clear deploy_steps
         node.refresh()
@@ -700,8 +702,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
             task = task_manager.TaskManager(self.context, node.uuid)
             task.process_event('deploy')
 
-            deployments.do_next_deploy_step(task, None,
-                                            self.service.conductor.id)
+            deployments.do_next_deploy_step(task, None)
 
             # Deploying should be complete without calling additional steps
             node.refresh()
@@ -727,7 +728,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
         task = task_manager.TaskManager(self.context, node.uuid)
         task.process_event('deploy')
 
-        deployments.do_next_deploy_step(task, 0, self.service.conductor.id)
+        deployments.do_next_deploy_step(task, 0)
 
         # Make sure we go to DEPLOYFAIL, clear deploy_steps
         node.refresh()
@@ -761,12 +762,47 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
 
         with task_manager.acquire(
                 self.context, node.uuid, shared=False) as task:
-            deployments.do_next_deploy_step(task, 0, mock.ANY)
+            deployments.do_next_deploy_step(task, 0)
 
         self._stop_service()
         node.refresh()
 
-        # Make sure we go to CLEANWAIT
+        # Make sure we go to DEPLOYWAIT
+        self.assertEqual(states.DEPLOYWAIT, node.provision_state)
+        self.assertEqual(tgt_prov_state, node.target_provision_state)
+        self.assertEqual(self.deploy_steps[0], node.deploy_step)
+        self.assertEqual(0, node.driver_internal_info['deploy_step_index'])
+        self.assertFalse(node.driver_internal_info['skip_current_deploy_step'])
+        mock_execute.assert_called_once_with(
+            mock.ANY, mock.ANY, self.deploy_steps[0])
+
+    @mock.patch('ironic.drivers.modules.fake.FakeDeploy.execute_deploy_step',
+                autospec=True)
+    def test_do_next_deploy_step_agent_busy(self, mock_execute):
+        # When a deploy step fails, go to DEPLOYWAIT
+        tgt_prov_state = states.ACTIVE
+
+        self._start_service()
+        node = obj_utils.create_test_node(
+            self.context, driver='fake-hardware',
+            provision_state=states.DEPLOYING,
+            target_provision_state=tgt_prov_state,
+            last_error=None,
+            driver_internal_info={'deploy_steps': self.deploy_steps,
+                                  'deploy_step_index': None,
+                                  'deployment_reboot': True},
+            clean_step={})
+        mock_execute.side_effect = exception.AgentInProgress(
+            reason='meow')
+
+        with task_manager.acquire(
+                self.context, node.uuid, shared=False) as task:
+            deployments.do_next_deploy_step(task, 0)
+
+        self._stop_service()
+        node.refresh()
+
+        # Make sure we go to DEPLOYWAIT
         self.assertEqual(states.DEPLOYWAIT, node.provision_state)
         self.assertEqual(tgt_prov_state, node.target_provision_state)
         self.assertEqual(self.deploy_steps[0], node.deploy_step)
@@ -795,7 +831,7 @@ class DoNextDeployStepTestCase(mgr_utils.ServiceSetUpMixin,
 
         with task_manager.acquire(
                 self.context, node.uuid, shared=False) as task:
-            deployments.do_next_deploy_step(task, 0, mock.ANY)
+            deployments.do_next_deploy_step(task, 0)
 
         self._stop_service()
         node.refresh()
@@ -843,7 +879,7 @@ class StoreConfigDriveTestCase(db_base.DbTestCase):
                           group='conductor')
         mock_swift.return_value.get_temp_url.return_value = 'http://1.2.3.4'
 
-        deployments._store_configdrive(self.node, b'foo')
+        deployments._store_configdrive(self.node, 'foo')
 
         mock_swift.assert_called_once_with()
         mock_swift.return_value.create_object.assert_called_once_with(
@@ -871,7 +907,7 @@ class StoreConfigDriveTestCase(db_base.DbTestCase):
                           group='conductor')
         mock_swift.return_value.get_temp_url.return_value = 'http://1.2.3.4'
 
-        deployments._store_configdrive(self.node, b'foo')
+        deployments._store_configdrive(self.node, 'foo')
 
         mock_swift.assert_called_once_with()
         mock_swift.return_value.create_object.assert_called_once_with(
@@ -898,7 +934,7 @@ class StoreConfigDriveTestCase(db_base.DbTestCase):
                           group='conductor')
         mock_swift.return_value.get_temp_url.return_value = 'http://1.2.3.4'
 
-        deployments._store_configdrive(self.node, b'foo')
+        deployments._store_configdrive(self.node, 'foo')
 
         mock_swift.assert_called_once_with()
         mock_swift.return_value.create_object.assert_called_once_with(
