@@ -20,7 +20,8 @@
 Test class for DRAC management interface
 """
 
-import mock
+from unittest import mock
+
 from oslo_utils import importutils
 
 import ironic.common.boot_devices
@@ -277,7 +278,8 @@ class DracManagementInternalMethodsTestCase(test_utils.BaseDracTest):
 
         self.assertEqual(0, mock_list_unfinished_jobs.call_count)
         self.assertEqual(0, mock_client.delete_jobs.call_count)
-        mock_validate_job_queue.assert_called_once_with(self.node)
+        mock_validate_job_queue.assert_called_once_with(
+            self.node, name_prefix="Configure: BIOS")
         mock_client.change_boot_device_order.assert_called_once_with(
             'OneTime', 'BIOS.Setup.1-1#BootSeq#NIC.Embedded.1-1-1')
         self.assertEqual(0, mock_client.set_bios_settings.call_count)
@@ -447,8 +449,8 @@ class DracManagementInternalMethodsTestCase(test_utils.BaseDracTest):
         self.assertEqual(0, mock_client.set_bios_settings.call_count)
         self.assertEqual(0, mock_client.commit_pending_bios_changes.call_count)
 
-    @mock.patch('time.time')
-    @mock.patch('time.sleep')
+    @mock.patch('time.time', autospec=True)
+    @mock.patch('time.sleep', autospec=True)
     @mock.patch.object(drac_mgmt, '_get_next_persistent_boot_mode',
                        spec_set=True, autospec=True)
     @mock.patch.object(drac_mgmt, '_get_boot_device', spec_set=True,
@@ -541,7 +543,8 @@ class DracManagementInternalMethodsTestCase(test_utils.BaseDracTest):
         self.assertEqual(0, mock_list_unfinished_jobs.call_count)
         self.assertEqual(0, mock_client.delete_jobs.call_count)
 
-        mock_validate_job_queue.assert_called_once_with(self.node)
+        mock_validate_job_queue.assert_called_once_with(
+            self.node, name_prefix="Configure: BIOS")
 
     @mock.patch.object(drac_job, 'validate_job_queue', spec_set=True,
                        autospec=True)
@@ -592,7 +595,8 @@ class DracManagementInternalMethodsTestCase(test_utils.BaseDracTest):
         self.assertEqual(0, mock_list_unfinished_jobs.call_count)
         self.assertEqual(0, mock_client.delete_jobs.call_count)
 
-        mock_validate_job_queue.assert_called_once_with(self.node)
+        mock_validate_job_queue.assert_called_once_with(
+            self.node, name_prefix="Configure: BIOS")
 
     @mock.patch.object(drac_mgmt, '_get_next_persistent_boot_mode',
                        spec_set=True, autospec=True)
