@@ -217,6 +217,10 @@ if not sushy:
         STATE_ABSENT='absent',
         VIRTUAL_MEDIA_CD='cd',
         VIRTUAL_MEDIA_FLOPPY='floppy',
+        APPLY_TIME_ON_RESET='on reset',
+        TASK_STATE_COMPLETED='completed',
+        HEALTH_OK='ok',
+        HEALTH_WARNING='warning'
     )
 
     sys.modules['sushy'] = sushy
@@ -231,6 +235,10 @@ if not sushy:
         type('MissingAttributeError', (sushy.exceptions.SushyError,), {}))
     sushy.exceptions.OEMExtensionNotFoundError = (
         type('OEMExtensionNotFoundError', (sushy.exceptions.SushyError,), {}))
+    sushy.exceptions.ServerSideError = (
+        type('ServerSideError', (sushy.exceptions.SushyError,), {}))
+    sushy.exceptions.BadRequestError = (
+        type('BadRequestError', (sushy.exceptions.SushyError,), {}))
     sushy.auth = mock.MagicMock(spec_set=mock_specs.SUSHY_AUTH_SPEC)
     sys.modules['sushy.auth'] = sushy.auth
 
